/* places.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported getExtension */

const Gettext = imports.gettext;

const { Clutter, Gio, GLib, GObject, Gtk, St } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;

const Dialog = imports.ui.dialog;
const Main = imports.ui.main;
const ModalDialog = imports.ui.modalDialog;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const Me = ExtensionUtils.getCurrentExtension();

const PlacesManager = Me.imports.placesManager;
const UserDialog = Me.imports.userDialog;

const _ = Gettext.domain(Me.metadata['gettext-domain']).gettext;

const BOOKMARKS_MAINMENU = 0;

const KEY_BOOKMARKS_POSITION = 'bookmarks-position';
const KEY_SHOW_PLACES = 'show-places';

const PANELMENU_BUTTON_ICON = 1;
const PANELMENU_BUTTON_TEXT = 2;

const PopupPlaceMenuItem = GObject.registerClass(
class PopupPlaceMenuItem extends PopupMenu.PopupImageMenuItem {
    _init(info) {
        super._init(info.name, info.icon);
        this.label.set({ x_expand: true, y_expand: true, y_align: Clutter.ActorAlign.CENTER });
        if (info.can_eject) {
            this._addButton('media-eject-symbolic', {
                content: 'ejectVolume',
                value: info.name,
                callback: () => { info.eject(); },
            });
        } else if (info.can_empty) {
            this._addButton('edit-clear-all-symbolic', {
                content: 'clearRecentDocuments',
                value: null,
                callback: () => { info.empty(); },
            });
        } else if (info.can_remove) {
            this._addButton('edit-clear-symbolic', {
                content: 'removeRecentDocument',
                value: info.name,
                callback: () => { info.remove(); },
            });
        }
        this._info = info;
    }

    _addButton(icon, params) {
        this.button = new St.Button({ style_class: 'button popup-menu-button' });
        this.button.set_child(new St.Icon({ icon_name: icon, style_class: 'popup-menu-icon' }));
        this.button.connect('clicked', event => {
            let dialog = new UserDialog.ModalUserDialog(params);
            Main.overview.hide();
            dialog.open();
            this.emit('activate', event);
        });
        this.add_child(this.button);
    }

    activate(event) {
        Main.overview.hide();
        this._info.launch(event.get_time());
        super.activate(event);
    }
});

const PopupPlacesMenuSection = class extends PopupMenu.PopupMenuSection {
    constructor(...params) {
        super();
        this._init(...params);
    }

    _init(manager) {
        this._placesManager = new PlacesManager[manager]();
        this._placesManagerChangedId = this._placesManager.connect('updated', this._reload.bind(this));
        this._reload();
    }

    _addPlaces(menu, kind) {
        let places = this._placesManager.get(kind);
        places.forEach(info => {
            menu.addMenuItem(new PopupPlaceMenuItem(info));
        });
    }

    _reload() {
        this.visible = this.numMenuItems > 0;
    }

    destroy() {
        this._placesManager.disconnect(this._placesManagerChangedId);
        this._placesManager.destroy();
        super.destroy();
    }
};

const PopupPlacesBookmarksMenuSection = class extends PopupPlacesMenuSection {
    _init() {
        this._settings = ExtensionUtils.getSettings();
        this._settingsChangedId = this._settings.connect('changed::%s'.format(KEY_BOOKMARKS_POSITION), this._reload.bind(this));
        super._init('PlacesBookmarksManager');
    }

    _reload() {
        let item;
        this.removeAll();
        this._addPlaces(this, 'special');
        if (this._settings.get_enum(KEY_BOOKMARKS_POSITION) === BOOKMARKS_MAINMENU) {
            this._addPlaces(this, 'default');
        } else {
            item = new PopupMenu.PopupSubMenuMenuItem(_("Bookmarks"), true);
            item.icon.icon_name = 'user-bookmarks';
            this._addPlaces(item.menu, 'default');
            if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        }
        item = new PopupMenu.PopupSubMenuMenuItem(_("Bookmarks (custom)"), true);
        item.icon.icon_name = 'user-bookmarks';
        this._addPlaces(item.menu, 'custom');
        if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        super._reload();
    }

    destroy() {
        this._settings.disconnect(this._settingsChangedId);
        super.destroy();
    }
};

const PopupPlacesRecentMenuSection = class extends PopupPlacesMenuSection {
    _init() {
        super._init('PlacesRecentManager');
    }

    _reload() {
        this.removeAll();
        if (this._placesManager.size > 0) {
            this._addPlaces(this, 'recent');
            let item = new PopupMenu.PopupSubMenuMenuItem(_("Last used documents"), true);
            item.icon.icon_name = 'document-open-recent';
            this._addPlaces(item.menu, 'recent-item');
            this.addMenuItem(item);
        }
        this.visible = this._placesManager.size > 0;
    }
};

const PopupPlacesVolumesMenuSection = class extends PopupPlacesMenuSection {
    _init() {
        super._init('PlacesVolumesManager');
    }

    _reload() {
        let item;
        this.removeAll();
        this._addPlaces(this, 'special');
        item = new PopupMenu.PopupSubMenuMenuItem(_("Network Drives"), true);
        item.icon.icon_name = 'network-server';
        this._addPlaces(item.menu, 'network');
        if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        item = new PopupMenu.PopupSubMenuMenuItem(_("Removable Devices"), true);
        item.icon.icon_name = 'media-removable';
        this._addPlaces(item.menu, 'device');
        if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        super._reload();
    }
};

const PlacesMenuButton = GObject.registerClass(
class PlacesMenuButton extends PanelMenu.Button {
    _init() {
        super._init(1.0, _("Places"), false);
        this.menu.actor.add_style_class_name('dschaerf-menu');
        this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
        this._icon = new St.Icon({ icon_name: 'folder-symbolic', style_class: 'system-status-icon' });
        this._hbox.add_child(this._icon);
        this._label = new St.Label({ text: _("Places"), y_expand: true, y_align: Clutter.ActorAlign.CENTER });
        this._hbox.add_child(this._label);
        this._hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
        this.add_actor(this._hbox);
        this._itemBookmarks = new PopupPlacesBookmarksMenuSection();
        this.menu.addMenuItem(this._itemBookmarks);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemVolumes = new PopupPlacesVolumesMenuSection();
        this.menu.addMenuItem(this._itemVolumes);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemRecent = new PopupPlacesRecentMenuSection();
        this.menu.addMenuItem(this._itemRecent);
        this._settings = ExtensionUtils.getSettings();
        this._settingsChangedId = this._settings.connect('changed::%s'.format(KEY_SHOW_PLACES), this._onChangedShowPlaces.bind(this));
        this._onChangedShowPlaces();
    }

    _onChangedShowPlaces() {
        let showPlaces = this._settings.get_enum(KEY_SHOW_PLACES);
        this._icon.visible = showPlaces !== PANELMENU_BUTTON_TEXT;
        this._label.visible = showPlaces !== PANELMENU_BUTTON_ICON;
    }

    _onDestroy() {
        this._settings.disconnect(this._settingsChangedId);
        super._onDestroy();
    }
});

function getExtension() {
    return Main.panel.addToStatusArea('PanelManagerPlacesMenu', new PlacesMenuButton(), 1, 'left');
}
