/* prefs.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported init buildPrefsWidget */

const Gettext = imports.gettext;

const { GLib, GObject, Gtk } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const _ = Gettext.domain(Me.metadata['gettext-domain']).gettext;

const SETTINGS_GENERALLY = [
    {
        type: 'combo',
        key: 'refresh-interval',
        label: _("Update interval in seconds"),
        tooltip: _("Sets the update interval for the temperature query of the sensors in seconds."),
        values: {
            '10': '10',
            '15': '15',
            '30': '30',
            '60': '60',
            '120': '120',
            '300': '300',
            '600': '600',
        },
    },
    {
        type: 'switcher',
        key: 'show-fahrenheit',
        label: _("Show temperature in Fahrenheit"),
        tooltip: _("When this option is activated, Fahrenheit is used as the temperature unit for the display instead of Celsius."),
    },
];

const TemperaturePrefsWidget = GObject.registerClass(
class TemperaturePrefsWidget extends Gtk.Notebook {
    _init() {
        super._init({ scrollable: true, margin: 15, width_request: 720, height_request: 480 });
        this._createPrefsWidget(SETTINGS_GENERALLY, _("Generally"));
        this.show_all();
    }

    _createPrefsWidget(prefs, text) {
        let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        prefs.forEach(pref => {
            let widget = this._createWidget(pref);
            if (widget) box.add(widget);
        });
        this.append_page(box, new Gtk.Label({ label: text }));
    }

    _createWidget(pref) {
        let settings = ExtensionUtils.getSettings(pref.schema ? pref.schema : null);
        let widget = null;
        widget = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
        widget.pack_start(new Gtk.Label({ label: _(pref.label), xalign: 0 }), true, true, 0);
        let prefAction;
        switch (pref.type) {
        case 'combo':
            prefAction = new Gtk.ComboBoxText();
            for (let value in pref.values) {
                prefAction.append(value, _(pref.values[value]));
            }
            prefAction.connect('changed', () => {
                settings.set_string(pref.key, prefAction.get_active_id());
            });
            prefAction.set_active_id(settings.get_string(pref.key));
            settings.connect('changed::%s'.format(pref.key), () => {
                prefAction.set_active_id(settings.get_string(pref.key));
            });
            break;
        case 'switcher':
            prefAction = new Gtk.Switch({ active: settings.get_boolean(pref.key) });
            prefAction.connect('notify::active', () => {
                settings.set_boolean(pref.key, prefAction.get_active());
            });
            settings.connect('changed::%s'.format(pref.key), () => {
                prefAction.set_active(settings.get_boolean(pref.key));
            });
            break;
        default: // Do nothing
        }
        if (pref.type !== 'switcher') prefAction.width_request = 240;
        if (pref.tooltip) prefAction.set_tooltip_text(_(pref.tooltip));
        widget.pack_end(prefAction, false, false, 0);
        return widget;
    }
});

function init(metadata) {
    ExtensionUtils.initTranslations();
}

function buildPrefsWidget() {
    return new TemperaturePrefsWidget();
}
